package tp.reseaux;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Insets;
import java.util.Arrays;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

public class FenetrePrincipale extends JFrame {
	
	String choixBoutonActionParDefaut = "Choissir une action";
	String choixBoutonActionCalculCRC = "Calcule CRC";
	String choixBoutonActionVerificationErreur = "Verification erreur";

	String[] tableauActionsBouton = {choixBoutonActionParDefaut,choixBoutonActionCalculCRC,choixBoutonActionVerificationErreur};
	JLabel labelChampsSaisieMots;
	JTextField champsSaisieMots;
	JComboBox<PolynomeGenerateur> comboChoixGenerateur;
	JComboBox<String> comboChoixAction;
	JTextArea txtAreaResultat;
	JButton boutonValider;
	Action actionBoutonValider;
	
	public FenetrePrincipale(){
		
		super("TP2 - CRC");
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		init();
		pack();
		setLocationRelativeTo(null);
		setVisible(true);
		
		
	}
	
	private void initActions() {
		actionBoutonValider = new ActionBoutonValider(this);
	}
	
	private void init() {
	
		initActions();
		initComposant();
	}
	
	private void initComposant() {
		JPanel panneauPrincipale = (JPanel) getContentPane();
		panneauPrincipale.setLayout(new BorderLayout());
		
		labelChampsSaisieMots = new JLabel("Saissir chaine binaire : ");
		labelChampsSaisieMots.setLabelFor(champsSaisieMots);
		champsSaisieMots = new JTextField(20);
		champsSaisieMots.setToolTipText("Chaine binaire exemple : 100110");
		comboChoixAction = new JComboBox<>(tableauActionsBouton);
		comboChoixGenerateur = new JComboBox<>(PolynomeGenerateur.values());
		boutonValider = new JButton(actionBoutonValider);
		txtAreaResultat = new JTextArea(10,10);
		txtAreaResultat.setEditable(false);

		txtAreaResultat.setMargin(new Insets(10, 10, 10, 10));

		JPanel panneauFormulaire = new JPanel();
        JScrollPane scrollPane = new JScrollPane(txtAreaResultat);

		panneauFormulaire.add(labelChampsSaisieMots);
		panneauFormulaire.add(champsSaisieMots);
		panneauFormulaire.add(comboChoixGenerateur);
		panneauFormulaire.add(comboChoixAction);
		panneauFormulaire.add(boutonValider);
		panneauPrincipale.add(panneauFormulaire,BorderLayout.NORTH);
		panneauPrincipale.add(scrollPane,BorderLayout.SOUTH);
		
	}
	

	public static void main(String[] args) {
		
		SwingUtilities.invokeLater(()->{
			
			new FenetrePrincipale();
		});
		
		
	}
	
	
	public void calculerCRC(String chaineBinaire, PolynomeGenerateur generateurChosit) {
        txtAreaResultat.setText("");
    	txtAreaResultat.setForeground(Color.BLUE);
		int degrePlusHaut = generateurChosit.getDegreLePlusHaut();
        String generateur = generateurChosit.getBinaire(); 
        String message = chaineBinaire + "0".repeat(degrePlusHaut);
        txtAreaResultat.append("Calcul de CRC \n");
        txtAreaResultat.append("Message binaire : " + chaineBinaire + "\n");
        txtAreaResultat.append("Generateur : " + generateurChosit.toString() + "\n");
        txtAreaResultat.append("Message après ajout des zéros : " + message +  "\n");
        String crc = divisionBinaire(message, generateur);
        txtAreaResultat.append("\n");
        txtAreaResultat.append("CRC : " + crc + "\n");
        txtAreaResultat.append("Message avec CRC : " + chaineBinaire + crc + "\n");
    }
	
	
	public void verifierErreur(String chaineBinaire, PolynomeGenerateur generateurChosit) {
        txtAreaResultat.setText("");
        int degrePlusHaut = generateurChosit.getDegreLePlusHaut();
        String generateur = generateurChosit.getBinaire();

        txtAreaResultat.append("Vérification d'erreur CRC \n");
        txtAreaResultat.append("Message binaire : " + chaineBinaire + "\n");
        txtAreaResultat.append("Generateur : " + generateurChosit.toString() + "\n");

        String reste = divisionBinaire(chaineBinaire, generateur);

        txtAreaResultat.append("\nReste final après division : " + reste  + "\n");

        if (reste.replace("0", "").isEmpty()) {
        	
        	txtAreaResultat.setForeground(Color.GREEN);
        	txtAreaResultat.append("Aucune erreur détectée ! Le message est correct. \n" );
        	
        } else {
        	txtAreaResultat.setForeground(Color.RED);
        	txtAreaResultat.append("Erreur détectée ! Le message est corrompu. \n");
        }
    }
    
    private String divisionBinaire(String message, String generateur) {
        int lenGen = generateur.length();
        char[] reste = message.substring(0, lenGen).toCharArray();

        txtAreaResultat.append("\n=== Début de la division binaire === \n");
        txtAreaResultat.append("Générateur : " + generateur);

        for (int i = lenGen; i <= message.length(); i++) {
        	  txtAreaResultat.append("\nÉtape " + (i - lenGen + 1) + " : ");
        	  txtAreaResultat.append("Reste actuel : " + new String(reste));
            
            if (reste[0] == '1') {
            	  txtAreaResultat.append(" XOR avec  " + generateur + "  →  ");
                for (int j = 0; j < lenGen; j++) {
                    reste[j] = (reste[j] == generateur.charAt(j)) ? '0' : '1'; // XOR
                }
                txtAreaResultat.append(new String(reste));
            } else {
            	  txtAreaResultat.append(" Pas de XOR (premier bit = 0)");
            }
            
            if (i < message.length()) {
                System.arraycopy(reste, 1, reste, 0, lenGen - 1);
                reste[lenGen - 1] = message.charAt(i);
                txtAreaResultat.append(" Décalage : " + new String(reste));
            }
        }

        txtAreaResultat.append("\n=== Fin de la division ===\n");
        String resteDiv = new String(reste).substring(1); 
        
        return resteDiv;
    }

	
	
	public String verifierErreur() {
		
		return null;
		
	}
	

	
	
	

}
