package tp.reseaux;

import java.util.Arrays;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public enum PolynomeGenerateur {
	
	CRC_4("4-2-1"),
	CRC_12("12-11-3-2-1-0"),
	CRC_16("16-15-2-0"),
	CR_CCITT("16-12-5-0"),
	CRC_32("32-26-23-22-16-12-11-10-8-7-5-4-1-0");
	
	private String degre;
	
	 PolynomeGenerateur(String degre) {
		 
		 this.degre = degre;
	}
	 
	 public String getDegre() {
		 
		 return degre;
		
	 }
	 
	 
	 public int[] getTabDegre() {
		 
		 
		 int[] tabDegres = Arrays.stream(degre.split("\\-"))
				 				 .mapToInt(Integer::parseInt)
				 				 .toArray();
		 return tabDegres;
	 }
	 
	 
	 
	 public int getDegreLePlusHaut() {
		 
			int degreLePlusHaut = Arrays.stream(getTabDegre())
					.max()
					.getAsInt();

		 return degreLePlusHaut;
	 }
	 
	 public String getBinaire() {
		 
	
			 int[] tabDegres = getTabDegre();
			 int tailleBinaire = getDegreLePlusHaut() + 1;
			 String[] tabBinnaire = new String [tailleBinaire];
			 Arrays.fill(tabBinnaire, "0");
			 
			 for(int index : tabDegres ) {
				 tabBinnaire[tailleBinaire-index-1] = "1";
			 }
			 
		     String chaineBinaire = Stream.of(tabBinnaire).collect(Collectors.joining(""));

			
			 return chaineBinaire;
			 
	 }

	
	 @Override
	 public String toString() {
		 
		 int[] tabDegres = getTabDegre();
		 						  
		 
		 StringBuilder polynome = new StringBuilder();
		 String symboleAdditionPlusEspacement;
		 String variableX;
		 
		 for(int i=0; i < tabDegres.length; i++ ) {
			
			 symboleAdditionPlusEspacement = (i==tabDegres.length - 1) ? "" : " + ";
			 variableX = (tabDegres[i]!=0) ? "x" + convertirExposant(tabDegres[i]) : "1" ;
			
			 polynome.append(variableX + symboleAdditionPlusEspacement);
			 
		 }	
		 
		 return polynome.toString();
		 		 
		 
	 }
	 
	 private  String convertirExposant(int n) {
	        String exposant = "⁰¹²³⁴⁵⁶⁷⁸⁹"; 
	        StringBuilder sb = new StringBuilder();
	        
	        String str = String.valueOf(n);
	        for (char c : str.toCharArray()) {
	            sb.append(exposant.charAt(c - '0')); 
	        }
	        
	        return sb.toString();
	    }

}
